using System;

namespace MyNamespace
{
	// ------- A complex single precision floating-point complex number
	public class ComplexF
	{
		public float r;		// real part
		public float i;		// imaginary part

		public ComplexF()	// initialize to zero
		{
			r = 0.0F;		// constructor logic
			i = 0.0F;
		}

		public ComplexF(float real, float imag)
		{
			r = real;		// constructor logic
			i = imag;
		}

//		public static void operator = (ComplexF a, ComplexF b)		// not allowed to overload =
//		{
//			a.r = b.r; a.i = b.i; return a;
//		}

		public static ComplexF operator & (ComplexF a, ComplexF b)	// use &= for = assignment
		{
			a.r = b.r;
			a.i = b.i;
			return a;
		}

		public static ComplexF operator + (ComplexF a, ComplexF b)	// can be used for a += b
		{
			ComplexF c = new ComplexF(a.r, a.i);
			c.r += b.r;
			c.i += b.i;
			return c;
		}

		public static ComplexF operator - (ComplexF a, ComplexF b)	// can be used for a -= b
		{
			ComplexF c = new ComplexF(a.r, a.i);
			c.r -= b.r;
			c.i -= b.i;
			return c;
		}

		public static ComplexF operator - (ComplexF a)
		{
			ComplexF c = new ComplexF(-a.r, -a.i);
			return c;
		}

		public static ComplexF operator * (ComplexF a, ComplexF b)	// can be used for a *= b
		{
			ComplexF c = new ComplexF();
			c.r = a.r * b.r - a.i * b.i;
			c.i = a.i * b.r + a.r * b.i;
			return c;
		}
		
		public static ComplexF operator / (ComplexF a, ComplexF b)	// can be used for a /= b
		{
			ComplexF c = new ComplexF();
			float r, den;
			if (Math.Abs(b.r) >= Math.Abs(b.i)) 
			{
				r = b.i / b.r;
				den = b.r + r * b.i;
				c.r = (a.r + r * a.i) / den;
				c.i = (a.i - r * a.r) / den;
			} 
			else 
			{
				r = b.r / b.i;
				den = b.i + r * b.r;
				c.r = (a.r * r + a.i) / den;
				c.i = (a.i * r - a.r) / den;
			}
			return c;
		}
		
		public static ComplexF operator / (ComplexF a, float b)		// can be used for a /= b
		{
			ComplexF c = new ComplexF();
			c.r = a.r / b;
			c.i = a.i / b;
			return c;
		}
		
		public static ComplexF operator * (ComplexF a, float b)		// can be used for a *= b
		{
			ComplexF c = new ComplexF();
			c.r = a.r * b;
			c.i = a.i * b;
			return c;
		}
		
		public static ComplexF operator * (float a, ComplexF b)		// can be used for a *= b
		{
			ComplexF c = new ComplexF();
			c.r = a * b.r;
			c.i = a * b.i;
			return c;
		}
		
		public float Real(ComplexF a)
		{
			return a.r;
		}

		public float Imag(ComplexF a)
		{
			return a.i;
		}
		public static ComplexF Set(float real, float imag)
		{
			ComplexF a = new ComplexF(real, imag);
			return a;
		}

		public static ComplexF Conjg(ComplexF a)
		{
			ComplexF c = new ComplexF();
			c.r = a.r;
			c.i = -a.i;
			return c;
		}

		public static float Abs(ComplexF a)
		{
			float x, y, ans, temp;
			x = Math.Abs(a.r);
			y = Math.Abs(a.i);
			if (x == 0.0F)
				ans = y;
			else if (y == 0.0F)
				ans = x;
			else if (x > y) 
			{
				temp = y / x;
				ans = (float)(x * Math.Sqrt(1.0F + temp * temp));
			} 
			else 
			{
				temp = x / y;
				ans = (float)(y * Math.Sqrt(1.0F + temp * temp));
			}
			return ans;
		} // Abs
	}
}
