using System;
using System.IO;					// for StreamWriter I/O to a file
using System.Text;
using System.Windows.Forms;			// for RichTextBox
using System.Reflection;			// for Assembly. ...

namespace MyNamespace
{
	/// <summary>
	///				Global services for several of Harry's programs
	///				Developed in TURBO Pascal 5.5 and 6.0, converted to Visual Basic 6
	///				and now in Visual C# .Net
	///
	/// ---------------- *** LEGAL NOTICE: *** ------------------
	///
	/// This program is free software; you can redistribute it and/or modify
	/// it under the terms of the GNU General Public License (GPL) as published
	/// by the Free Software Foundation. cf. the file COPYING.txt.
	///
	/// ------------ *** end of legal notice *** ------------------
	/// </summary>

	public class Global
	{
		public Global() // constructor logic
		{
			command = "";
			helpUp = 0;
			myURL = "hjsmithh@sbcglobal.net  http://www.geocities.com/hjsmithh/";
			programURL = "http://www.geocities.com/hjsmithh/download.html";
			runFormUp = false;
			GetAssemblyAttributes();
			nL = Environment.NewLine;
			nL2 = nL + nL;
			nLc = nL;						// line continuation mark for WriteT nL or "-" + nL
			logg = false;
			logFile = "";
			muErrRep = true;
		}

		public static bool IsInt32(string sInt32) // is input string a good 32-bit signed integer?
		{
			int iInt32;
			try
			{
				iInt32 = System.Int32.Parse(sInt32);
				return true;
			}
			catch // (FormatException ex) and (OverflowException ex)
			{
				return false;
			}
		}

		public static void Write(string message) // write a short message to outS
		{
			Write(outS, null, message);
		}

		public static void WriteLn() // write a linefeed to outS
		{
			WriteLn(outS, null, "");
		}

		public static void WriteLn(string message) // write a short message and a linefeed to outS
		{
			WriteLn(outS, null, message);
		}

		public static void WriteLn(RichTextBox outB, StreamWriter outF) // Write linefeed
		{
			Write(outB, outF, nL);
		}

		public static void WriteLn(RichTextBox outB, StreamWriter outF, string st) // Write st plus linefeed
		{
			Write(outB, outF, st + nL);
		}

		public static void WriteLn2() // Write 2 linefeeds to outS
		{
			Write(outS, null, nL2);
		}

		public static void WriteLn2(string st) // Write st plus 2 linefeeds to outS
		{
			Write(outS, null, st + nL2);
		}

		public static void WriteLn2(RichTextBox outB, StreamWriter outF) // Write 2 linefeeds
		{
			Write(outB, outF, nL2);
		}

		public static void WriteLn2(RichTextBox outB, StreamWriter outF, string st) // Write st plus 2 linefeeds
		{
			Write(outB, outF, st + nL2);
		}

		public static void WriteSLn(string st) // Write SingleSpace, st, and linefeed to outS
		{
			SingleSpace(outS);
			Write(outS, null, st + nL);
		}

		public static void WriteDLn(string st) // Write DoubleSpace, st, and linefeed to outS
		{
			DoubleSpace(outS);
			Write(outS, null, st + nL);
		}

		public static void WriteDLn2(string st) // Write DoubleSpace, st, and 2 linefeeds to outS
		{
			DoubleSpace(outS);
			Write(outS, null, st + nL2);
		}

		public static void WriteDLn(RichTextBox outB, StreamWriter outF, string st) // Write DoubleSpace, st, and linefeed
		{
			DoubleSpace(outB);
			Write(outB, outF, st + nL);
		}

		public static void Write(RichTextBox outB, StreamWriter outF, string st) // print string to RichTextBox or to a file
		{
			if (st == "") return;
			if (outF != null)
			{
				try { outF.Write(st); }
				catch (Exception ex) { ReportError("Write", ex.Message); }
			}
			if (outB != null)
			{
				outB.AppendText(st);
				if (st[(st.Length - 1)] == '\n')	// if a new line, move to bottom/end of output text box
				{
					Global.runTextOut.Select(Global.runTextOut.Text.Length, 0);
					Global.runTextOut.Focus();
				}
				if (logg && (outB == outS))			// only log output to Run form textbox
				{
					try { outL.Write(st); }
					catch { }
				}
				if (outB.Text.Length > 200000)		// clear top of screen after about 50 full pages
				{
					int i = outB.Text.IndexOf("\n", 4000);
					outB.Text = outB.Text.Substring(i + 1);	// delete partial line
				}
			}
		} // Write

		public static void WriteT(string st)
		// WriteT with 0 tot and 0 width to outS
		{
			int tot = 0;
			SingleSpace();
			WriteT(outS, null, st, ref tot, 0);
		}

		public static void WriteT(string st, ref int tot)
		// WriteT with 0 width to outS
		{
			WriteT(outS, null, st, ref tot, 0);
		}

		public static void WriteT(RichTextBox outB, StreamWriter outF, string st)
		// WriteT with 0 tot and 0 width
		{
			int tot = 0;
			SingleSpace(outB);
			WriteT(outB, outF, st, ref tot, 0);
		}

		public static void WriteT(RichTextBox outB, StreamWriter outF, string st, ref int tot)
		// WriteT with 0 width
		{
			WriteT(outB, outF, st, ref tot, 0);
		}

		public static void WriteTLn()
		// WriteTLn with "", 0 tot and 0 width to outS
		{
			Write(outS, null, nL);
		}

		public static void WriteTLn(string st)
		// WriteTLn with 0 tot and 0 width to outS
		{
			int tot = 0;
			SingleSpace();
			if (st != "") WriteT(outS, null, st, ref tot, 0);
			Write(outS, null, nL);
		}

		public static void WriteTLn(string st, ref int tot)
		// WriteT and linefeed to outS
		{
			if (st != "") WriteT(outS, null, st, ref tot, 0);
			Write(outS, null, nL);
			tot = 0;
		}

		public static void WriteTLn(RichTextBox outB, StreamWriter outF, string st)
		// WriteT and linefeed with 0 tot and 0 width
		{
			int tot = 0;
			SingleSpace(outB);
			if (st != "") WriteT(outB, outF, st, ref tot, 0);
			Write(outB, outF, nL);
		}

		public static void WriteTLn(RichTextBox outB, StreamWriter outF, string st, ref int tot)
		// WriteT and linefeed with 0 width
		{
			if (st != "") WriteT(outB, outF, st, ref tot, 0);
			Write(outB, outF, nL);
			tot = 0;
		}

		public static void WriteTLn(RichTextBox outB, StreamWriter outF, string st, ref int tot, int width)
		// WriteT and linefeed
		{
			if (st != "") WriteT(outB, outF, st, ref tot, width);
			Write(outB, outF, nL);
			tot = 0;
		}

		public static void WriteTLn2(string st)
		// WriteTLn2 with 0 tot and 0 width to outS
		{
			int tot = 0;
			SingleSpace();
			if (st != "") WriteT(outS, null, st, ref tot, 0);
			Write(outS, null, nL2);
		}

		public static void WriteTLn2(RichTextBox outB, StreamWriter outF, string st)
		// WriteTLn2 with 0 tot and 0 width
		{
			int tot = 0;
			SingleSpace(outB);
			if (st != "") WriteT(outB, outF, st, ref tot, 0);
			Write(outB, outF, nL2);
		}

		public static void WriteTLn2(RichTextBox outB, StreamWriter outF, string st, ref int tot, int width)
		// WriteT and a double linefeed after
		{
			if (st != "") WriteT(outB, outF, st, ref tot, width);
			Write(outB, outF, nL2);
			tot = 0;
		}

		public static void WriteTD(RichTextBox outB, StreamWriter outF, string st)
		// force double space and WriteT with 0 tot and 0 width
		{
			int tot = 0;
			SingleSpace(outB);
			WriteT(outB, outF, st, ref tot, 0);
		}

		public static void WriteTDLn(string st)
		// WriteT with a guaranteed double linefeed before and single linefeed after to outS
		{
			DoubleSpace(outS);
			WriteTLn(outS, null, st);
		}

		public static void WriteTDLn(RichTextBox outB, StreamWriter outF, string st)
		// WriteT with a guaranteed double linefeed before and single linefeed after
		{
			DoubleSpace(outB);
			WriteTLn(outB, outF, st);
		}

		public static void WriteT(RichTextBox outB, StreamWriter outF, string st, ref int tot, int width)
		// output String and linefeed every 60, 80 or width characters
		// tot = characters on line so far
		{
			int i;
			int j;
			int iL;
			int n;
			string stE;
			if (width > 0)
				n = width;
			else
			{
				if (outB == null) n = 60;	// 80 for screen, 60 for files
				else n = 80;
			}
			i = 1;
			j = n - tot;
			iL = st.Length;
			while (iL > 0)
			{
				stE = "";
				if (tot >= n)
				{
					stE = nLc;				// format n characters per line
					j = n;
					tot = 0;
				}
				if (iL <= j)
				{
					Write(outB, outF, stE + st.Substring(i - 1));	// built for speed
					tot += iL;									// don't call Write too often
					iL = 0;
				}
				else
				{
					Write(outB, outF, stE + st.Substring(i - 1, j));
					i += j;
					iL -= j;
					j = n;
					tot = n;
				}
			}
		} // WriteT

		public static void MuWriteErr(string st) // write error statement to outS and set error flag
		{
			muErr = true;
			if (muErrRep) WriteDLn(st);
		}

		public static void Write(float x)				// Write x to TextOut
		{
			Global.runTextOut.Text += x.ToString();
		}

		public static void SingleSpace() // guarantee a single line space in outS
		{
			SingleSpace(outS);
		}

		public static void SingleSpace(RichTextBox outB) // guarantee a single line space in output box
		{
			int iL = outB.Text.Length;
			if (iL > 0)
			{
				if (outB.Text.Substring(iL - 1) != "\n") WriteLn(outB, null);
			}
		}

		public static void DoubleSpace(RichTextBox outB) // guarantee a double line space in output box
		{
			int iL = outB.Text.Length;
			if (iL > 0)
			{
				if (outB.Text.Substring(iL - 1) != "\n") WriteLn(outB, null);
				if (outB.Text.Substring(outB.Text.Length - 2) != "\n\n") WriteLn(outB, null);
			}
		}

		public static void DoubleSpace() // guarantee a double line space in outS
		{
			DoubleSpace(outS);
		}

		public static void GetAssemblyAttributes() // and version
		{
			system = Environment.GetEnvironmentVariable("SYSTEM");
			if (system == null)
				system = "";
			else
				system += Environment.NewLine;
			system += Environment.GetEnvironmentVariable("COMPUTERNAME");
			system += " by ";
			system += Environment.GetEnvironmentVariable("USERNAME");
			if (system == null || system == "")
				system = "<Computer system name not set>"; foreach (object attr in Assembly.GetExecutingAssembly().GetCustomAttributes(false))
			{
				AssemblyTitleAttribute aTitle = attr as AssemblyTitleAttribute;
				if (aTitle != null) title = aTitle.Title;

				AssemblyDescriptionAttribute aDescription = attr as AssemblyDescriptionAttribute;
				if (aDescription != null) description = aDescription.Description;

				AssemblyConfigurationAttribute aConfiguration = attr as AssemblyConfigurationAttribute;
				if (aConfiguration != null) configuration = aConfiguration.Configuration;

				AssemblyCompanyAttribute aCompany = attr as AssemblyCompanyAttribute;
				if (aCompany != null) company = aCompany.Company;

				AssemblyProductAttribute aProduct = attr as AssemblyProductAttribute;
				if (aProduct != null) product = aProduct.Product;

				AssemblyCopyrightAttribute aCopyright = attr as AssemblyCopyrightAttribute;
				if (aCopyright != null) copyright = aCopyright.Copyright;

				AssemblyTrademarkAttribute aTrademark = attr as AssemblyTrademarkAttribute;
				if (aTrademark != null) trademark = aTrademark.Trademark;
			}
			Version aVersion = Assembly.GetExecutingAssembly().GetName().Version;
			int versionMajor = aVersion.Major;
			int versionMinor = aVersion.Minor;
			int versionBuild = aVersion.Build;				// the number of days since January 1, 2000 local time
			int versionRevision = aVersion.Revision;		// the number of seconds since midnight local time, divided by 2
			version = "C# Forms Version " + aVersion.ToString();
			name1 = product;
			name2 = description;
			author = copyright;
			address = trademark;
			string[] fields;
			fields = title.Split(' ');				// convert the string to an array each, number is separated by a space
			pName = fields[0].ToString();
		} // GetAssemblyAttributes

		public static bool OpenRead(out StreamReader inF, string fName, out string dir)
		// open file for Reading from CurrentDirectory or homeDir
		{
			string stEx1;
			string stEx2 = "";
			string savDir = Environment.CurrentDirectory.ToString();
			if (!quiet) DoubleSpace();
			dir = savDir;
			inF = null;
			if (OpenReadFile(out inF, fName, out stEx1))
			{
				if (!quiet)
				{
					WriteDLn("File " + fName + " opened for reading");
					WCurDir(fName);
				}
				return true;
			}
			if (savDir != homeDir)
			{
				dir = homeDir;
				Directory.SetCurrentDirectory(homeDir);
				if (OpenReadFile(out inF, fName, out stEx2))
				{
					if (!quiet)
					{
						WriteLn("File " + fName + " opened for reading");
						WCurDir(fName);
					}
					Directory.SetCurrentDirectory(savDir);
					return true;
				}
				Directory.SetCurrentDirectory(savDir);
			}
			dir = "";
			WriteLn("Cannot open file " + fName + " for input");
			WriteTLn("Not in current directory " + savDir);
			ReportError("OpenRead", stEx1);			// file not found most likely
			if (savDir != homeDir)
			{
				WriteTLn("Not in home directory " + homeDir);
				ReportError("OpenRead", stEx2);	// file not found most likely
			}
			return false;
		} // OpenRead

		public static bool OpenReadFile(out StreamReader inF, string fName, out String stEx)
		// open file for Reading from CurrentDirectory
		{
			inF = null;
			stEx = "";
			try
			{
				inF = new StreamReader(fName); // open fName for input as inF
				return true;
			}
			catch (Exception ex)
			{
				stEx = ex.Message.ToString();
				return false;
			}
		} // OpenReadFile

		public static void ReportError(string who, string errorMessage)	// report IOResult etc. error, errorMessage = caught.Message
		{
			GC.Collect();	// force garbage collection of all generations
			muErr = true;
			if (errorMessage == "") return;
			if (muErrRep) WriteTLn(who + ": " + errorMessage);
		}

		public static void WCurDir(string fName)
		// write current disk directory with final \
		// using Environment.CurrentDirectory.ToString(): C:\ already has the \ but non-root directories do not
		{
			// this can garble some output, but good info
			string cD = Directory.GetCurrentDirectory();
			if (cD.Substring(cD.Length - 1) != @"\") cD += @"\";
			if (fName.Length > 1 && fName[1] == ':')
			{
				WriteTLn("Directory name = " + cD);
			}
			else
			{
				WriteTLn("Full name = " + cD + fName);
			}
		}

		public static void ClearLog()	// clear Log file
		{
			DateTimePicker dateTime = new System.Windows.Forms.DateTimePicker();
			string dateTimeString = dateTime.Value.ToString();
			bool logS = logg;	// save Log status in a local
			if (!quiet) DoubleSpace();
			if (logg)
			{
				string st = "Log file " + logFile + " Closed " + dateTimeString;
				if (!quiet)
				{
					WriteLn(st);
					outL.WriteLine();
				}
				WriteLn(null, outL);
				outL.Close();		// close the file
				logg = false;
			}
			logFile = "";
			CalLFile(ref logFile);	// get log file name, it may have changed
			// open logFile for output as outL, clear the file, outL gotten by open routine
			if (ChkWrite(out outL, logFile, ""))
			{
				logg = true;		// set to get message to the file
				string st = "Log file " + logFile + " Cleared " + dateTimeString;
				if (!quiet)
				{
					WriteLn(st);
					WCurDir(logFile);
				}
				else
				{
					outL.WriteLine(st);
				}
				outL.WriteLine(Global.title + ", " + Global.version);
				if (Global.system != "") outL.WriteLine("Run on: " + Global.system);
				if (!quiet)
				{
					WriteLn();
				}
				else
				{
					outL.WriteLine();
				}
				if (!logS)
				{
					outL.Close();	// close the file
					logg = false;
					logFile = "";			// clear name when it is closed, not if its open
				}
			}
		} // ClearLog

		public static void ToggleLogg() //Toggle Logging of output screen to Log File
		{
			DateTimePicker dateTime = new System.Windows.Forms.DateTimePicker();
			string dateTimeString = dateTime.Value.ToString();
			if (!quiet) DoubleSpace();
			if (logg)
			{
				string st = "Log file " + logFile + " Closed " + dateTimeString;
				if (!quiet)
				{
					WriteLn(st);
					outL.WriteLine();
				}
				outL.Close();	// close the file
				logFile = "";			// allow for getting a new log file name next open
				logg = false;
			}
			else
			{
				CalLFile(ref logFile); // get log file name
				bool check;
				check = ChkAppend(ref outL, logFile);
				if (check) // open logFile for Append as outL
				{
					logg = true;
					string st = "Log file " + logFile + " Opened for Append " + dateTimeString;
					if (!quiet)
					{
						WriteDLn(st);
						WCurDir(logFile);
						outL.WriteLine(Global.title + ", " + Global.version);
						if (Global.system != "") outL.WriteLine("Run on: " + Global.system);
					}
				}
				else
				{
					logFile = ""; // file did not open
					return;
				}
			}
			Global.objRunForm.SetLogCaption(); // call sub in Run form
		} // ToggleLogg

		public static bool OpenWrite(out StreamWriter outF, string fName) // open file for Writing
		{
			outF = null; // outF must be assigned to before control leaves the current method
			try
			{
				// open fName for output as outF
				outF = new StreamWriter(fName, false, Encoding.ASCII); // Appending = false
				if (!quiet)
				{
					WriteDLn("File " + fName + " opened for writing");
					WCurDir(fName);
				}
				return true;
			}
			catch (Exception ex)
			{
				WriteDLn("Cannot open file " + fName + " for output");
				WCurDir(fName);
				ReportError("OpenWrite", ex.Message);
				return false;
			}
		} // OpenWrite

		public static bool ChkWrite(out StreamWriter outF, string fName, string st)
		// check start write of file
		{
			if (!OpenWrite(out outF, fName))
			{
				if (MakeReady() == DialogResult.OK)
				{
					if (!OpenWrite(out outF, fName))
					{
						WriteLn("Aborting file write...");
						return false;
					}
				}
			}
			if (!quiet) WriteTLn("Writing file " + fName + st);
			return true;
		}

		public static bool OpenAppend(ref StreamWriter outF, string fName)	// open file for Writing
		{
			try
			{
				// open fName for Append as outF
				outF = new StreamWriter(fName, true, Encoding.ASCII);		//Appending = true
				return true;
			}
			catch (Exception ex)
			{
				WriteDLn("Cannot open file " + fName + " for output");
				WCurDir(fName);
				ReportError("OpenAppend", ex.Message);
				return false;
			}
		}

		public static bool ChkAppend(ref StreamWriter outF, string fName)
		// check start Append to a file
		{
			if (!OpenAppend(ref outF, fName))
			{
				if (MakeReady() == DialogResult.OK)
				{
					if (!OpenAppend(ref outF, fName))
					{
						WriteLn("Aborting file write...");
						return false;
					}
				}
			}
			return true;
		}

		public static DialogResult MakeReady() // pause for operator to make file/drive ready
		{
			DialogResult key = MessageBox.Show(Global.pName + ": Make file/drive ready and click \"OK\"\n" +
								StringOf(29, " ") + "or \"Cancel\"",
				Global.title,
				MessageBoxButtons.OKCancel,
				MessageBoxIcon.Exclamation);
			// pause ----------
			if (key != DialogResult.OK) WriteLn("I/O operation aborted by operator!");
			return key;
		}

		public static string StringOf(int n, string stIn) // repeat stIn, n times
		{
			string stOut = "";
			for (int i = 0; i < n; i++)
			{
				stOut += stIn;
			}
			return stOut;
		}

		public static void CalLFile(ref string fName) // establish log file name
		{
			if (fName == "")
				fName = logFileB;
		}

		public static string system;				// EnvironmentVariable("SYSTEM")
		public static string title;					// this is an example of retrieving AssemblyInfo
		public static string description;			// see GetAssemblyAttributes() above
		public static string configuration;
		public static string company;
		public static string product;
		public static string copyright;
		public static string trademark;

		public static string name1;					// = product
		public static string name2;					// = description
		public static string version;				// = "C# Version x.x.xxxx.xxxxx"
		public static string author;				// = copyright
		public static string address;				// = trademark

		public static bool logg;					// true if Logging screen output to FILE
		public static bool muErr;					// true if error occurred
		public static bool muErrRep;				// true if error reports are desired
		public static bool parmsDone;				// true if command line parameters have been processed
		public static bool quiet;					// true if "quiet" mode is on
		public static bool runFormUp;				// the Run Form is up and running
		public static bool tExitFlag;				// command to totally exit the program
		public static bool tQuitReq;				// request to totally quit the program
		public static int helpUp;					// number of Help form up and running
		public static RichTextBox inS;				// input Screen source of commands
		public static RichTextBox outS;				// screen Output destination
		public static RichTextBox runTextIn;		// main input window on Run form
		public static RichTextBox runTextOut;		// main output window on Run form
		public static RunForm objRunForm;			// the RunForm object for function calls
		public static StreamWriter outL;			// log file output #
		public static string command;				// commands from command line arguments
		public static string homeDir;				// initial disk directory when program started
		public static string logFile;				// file name of log file or "" if not logging
		public static string logFileB;				// base log file name
		public static string myURL;					// my e-mail address and homepage URL
		public static string nL;					// one Environment.NewLine \n or \r or \r\n
		public static string nL2;					// nL + nL
		public static string nLc;					// line continuation mark for WriteT nL or "-" + nL
		public static string pName;					// from title
		public static string programURL;			// URL of related web page
	}
}
