using System;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace MyNamespace
{
	/// <summary>
	///		HelpForm for several of Harry's programs
	///
	/// ---------------- *** LEGAL NOTICE: *** ------------------
	///
	/// This program is free software; you can redistribute it and/or modify
	/// it under the terms of the GNU General Public License (GPL) as published
	/// by the Free Software Foundation. cf. the file COPYING.txt.
	///
	/// ------------ *** end of legal notice *** ------------------
	/// </summary>

	public partial class HelpForm : Form
	{
		public HelpForm()
		{
			InitializeComponent();

			helpTextOut = richTextOut;
			helpTextOut.Clear();
			findSt = "";
		}

		public void Help()	// -------- Begin Main program Help
		{
			if (Global.helpUp >= 3) { Dispose(true); return; }	// allow at most 3 Help Forms up
			Global.helpUp++;
			this.Cursor = Cursors.Arrow;
			this.Text = Global.title + " - Help";
			labelTitle.Text = Global.title + " Help";
			HelpFile = Global.pName + "Help.txt";
			ReadHelp();
			RunForm.FocusBothUs();
			helpTextOut.Select(0, 0);			// move cursor to top
			helpTextOut.Focus();				// and focus
			this.Width = RunForm.helpWidth;		// Resize the form
			this.Height = RunForm.helpHeight;
		}

		private void ReadHelp()
		{
			StreamReader readHelpFile = null;
			string line;

			try
			{
				helpTextOut.Text = "";					// clear text output window
				PLn("");
				PLn("Reading Help File: " + HelpFile);
				string dir;
				if (!Global.OpenRead(out readHelpFile, HelpFile, out dir)) // open the file
				{
					PLn("File not found: " + HelpFile);	// file not found
					return;
				}
				helpTextOut.Text = "";					// clear text output window again
				line = readHelpFile.ReadLine();			// read first line of file
				while (line != null)					// loop until end of file
				{
					PLn("  " + line);					// output a line of Help
					line = readHelpFile.ReadLine();		// read in next line
				}
			}
			catch (Exception ex)
			{
				PLn("ReadHelp: " + ex.Message);			// read error
				Global.ReportError("ReadHelp", ex.Message);
			}
			finally
			{
				if (readHelpFile != null)
				{
					readHelpFile.Close();				// close the file
				}
			}
		} // ReadHelp

		private void Prn(string st) // print string to textOut window
		{
			helpTextOut.AppendText(st);
		}

		private void PLn(string st) // print string ... and new line to textOut
		{
			Prn(st + "\n");
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Global.helpUp--;
			Dispose(true);
		}

		private void HelpForm_Resize(object sender, EventArgs e)
		{
			int jTop = labelTitle.Top + labelTitle.Height + 5;
			this.richTextOut.SetBounds(1, jTop, this.ClientRectangle.Width - 2, this.ClientRectangle.Height - jTop - 1);
		}

		private void richTextOut_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Escape) // ESC key pressed so Exit Help
			{
				Global.helpUp--;
				Dispose(true);
			}
			if (e.KeyCode == Keys.F2)	// F2 key pressed so do a find going up
			{
				buttonFindUp_Click(sender, e);
			}
			if (e.KeyCode == Keys.F3 || e.KeyCode == Keys.Return)	// F3 key or Return pressed so do a normal find
			{
				buttonFind_Click(sender, e);
			}
			if (e.KeyCode == Keys.F)	// F key pressed so copy selected text to textBoxFind
			{
				if (e.Control)
				{
					string st = helpTextOut.SelectedText;	// only copy up to first newline
					int i = st.IndexOf("\n");
					if (i >= 0) st = st.Substring(0, i);
					textBoxFind.Text = st;
					textBoxFind.Select(0, st.Length);		// move cursor found location
					textBoxFind.Focus();					// and focus
				}
			}
		} // richTextOut_KeyDown

		internal class Helpers
		{
			[Flags]
			public enum PlaySoundFlags : int
			{
				SND_SYNC = 0x0000,			/* play synchronously (default) */
				SND_ASYNC = 0x0001,			/* play asynchronously */
				SND_NODEFAULT = 0x0002,		/* silence (!default) if sound not found */
				SND_MEMORY = 0x0004,		/* pszSound points to a memory file */
				SND_LOOP = 0x0008,			/* loop the sound until next sndPlaySound */
				SND_NOSTOP = 0x0010,		/* don't stop any currently playing sound */
				SND_NOWAIT = 0x00002000,	/* don't wait if the driver is busy */
				SND_ALIAS = 0x00010000,		/* name is a registry alias */
				SND_ALIAS_ID = 0x00110000,	/* alias is a predefined ID */
				SND_FILENAME = 0x00020000,	/* name is file name */
				SND_RESOURCE = 0x00040004	/* name is resource name or atom */
			}

			[DllImport("winmm.dll")]
			public static extern bool PlaySound(string szSound, IntPtr hMod, PlaySoundFlags flags);

			[DllImport("kernel32.dll")]
			public static extern bool Beep(int freq, int dur);
		}

		public class Sound
		{
			public static void Play(string strFileName)
			{
				try
				{
					Helpers.PlaySound(strFileName, IntPtr.Zero, Helpers.PlaySoundFlags.SND_FILENAME | Helpers.PlaySoundFlags.SND_ASYNC);
				}
				catch
				{
					// DllImport("winmm.dll") file not found or no PlaySound entry point in the file
					// so generate no sound
				}
			}
		}

		public int FindMyText(string searchText, int searchStart, int searchEnd)
		{
			// Initialize the return value to false by default.
			int returnValue = -1;

			// Ensure that a search string and a valid starting point are specified.
			if (searchText.Length > 0 && searchStart >= 0)
			{
				// Ensure that a valid ending value is provided.
				if (searchEnd > searchStart || searchEnd == -1)
				{
					// Obtain the location of the search string in the richTextBox.
					int indexToText = richTextOut.Find(searchText, searchStart, searchEnd, RichTextBoxFinds.None);
					// Determine whether the text was found in richTextBox1.
					if (indexToText >= 0)
					{
						// Return the index to the specified search text.
						returnValue = indexToText;
					}
				}
			}
			return returnValue;
		} // FindMyText

		public int FindMyTextUp(string searchText, int searchStart, int searchEnd)
		{
			// Initialize the return value to false by default.
			int returnValue = -1;

			// Ensure that a search string and a valid starting point are specified.
			if (searchText.Length > 0 && searchStart >= 0)
			{
				// Ensure that a valid ending value is provided.
				if (searchEnd > searchStart || searchEnd == -1)
				{
					// Obtain the location of the search string in the richTextBox.
					int indexToText = richTextOut.Find(searchText, searchStart, searchEnd, RichTextBoxFinds.Reverse);
					// Determine whether the text was found in richTextBox1.
					if (indexToText >= 0)
					{
						// Return the index to the specified search text.
						returnValue = indexToText;
					}
				}
			}
			return returnValue;
		} // FindMyTextUp

		private void buttonFind_Click(object sender, EventArgs e)
		{
			int found = -1;
			helpTextOut.Focus();				// focus on output text
			if (textBoxFind.Text != "")
				findSt = textBoxFind.Text;
			int length = findSt.Length;
			if (length == 0)
				return;
			int scope = helpTextOut.Text.Length;
			int start = helpTextOut.SelectionStart;
			int begin = Math.Min(start + 1, scope);
			if (begin != scope)
				found = FindMyText(findSt, begin, scope);
			if (found < 0)
			{
				int end = Math.Min(start + length, scope);
				found = FindMyText(findSt, 0, end);
				if (found >= 0)
				{
					Sound.Play("Wrap.wav");
				}
				else
				{
					Sound.Play("NotFound.wav");
				}
			}
			else
			{
				TryBeep(277, 50);					// beep Frequency 277Hz, Length 50ms
			}										// 277 is middle C#
			if (found >= 0)
			{
				helpTextOut.Select(found, length);	// move cursor found location
				helpTextOut.Focus();				// and focus
			}
		} // buttonFind_Click

		private void buttonFindUp_Click(object sender, EventArgs e)
		{
			int found = -1;
			helpTextOut.Focus();				// focus on output text
			if (textBoxFind.Text != "")
				findSt = textBoxFind.Text;
			int length = findSt.Length;
			if (length == 0)
				return;
			int scope = helpTextOut.Text.Length;
			int start = helpTextOut.SelectionStart + length;
			int end = Math.Max(start - 1, 0);
			if (end != 0)
				found = FindMyTextUp(findSt, 0, end);
			if (found < 0)
			{
				int begin = Math.Max(start - length, 0);
				found = FindMyTextUp(findSt, begin, scope);
				if (found >= 0)
				{
					Sound.Play("Wrap.wav");
				}
				else
				{
					Sound.Play("NotFound.wav");
				}
			}
			else
			{
				TryBeep(277 * 2, 50);				// beep Frequency 554Hz, Length 50ms
			}										// 554 is one octave above middle C#
			if (found >= 0)
			{
				helpTextOut.Select(found, length);	// move cursor found location
				helpTextOut.Focus();				// and focus
			}
		} // buttonFindUp_Click

		private void TryBeep(int freq, int dur)
		{
			try
			{
				Helpers.Beep(freq, dur);			// beep freq, duration
			}
			catch
			{
				// DllImport("kernel32.dll") file not found or no Beep entry point in the file
				// so no beep
			}
		}

		private void HelpForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			Global.helpUp--;
		}

		private readonly RichTextBox helpTextOut;
		private string HelpFile;				// file name of Help file
		private string findSt;					// string to find
	}
}
