using System;
using System.Windows.Forms;

namespace MyNamespace
{
	public partial class RunForm : Form
	{
		public RunForm()
		{
			InitializeComponent();

			Global.runTextOut = richTextOut;
			Global.runTextOut.Clear();
			Global.runTextIn = richTextIn;
			Global.runTextIn.Clear();
			Global.objRunForm = this;
			this.Width = runWidth;				// Resize the form
			this.Height = runHeight;
			this.CenterToScreen();
		}

		public void Run(int it, StartupForm callerForm)
		// -------- Begin Main program Run for input and output to control calculations
		{
			if (Global.runFormUp) { Dispose(true); return; }
			Global.tExitFlag = false;
			tQuitFlag = true;
			Global.runFormUp = true;
			Global.programURL = "http://www.geocities.com/hjsmithh/download.html";
			this.Text = Global.title + " - Run";
			objStartupForm = callerForm;
			if (Global.homeDir == null) Global.homeDir = Environment.CurrentDirectory.ToString();
			Global.outS = Global.runTextOut;
			Global.inS = Global.runTextIn;
			Global.runTextOut.Clear();
			Global.runTextIn.Clear();				// clear Input text box first time
			Global.objRunForm = this;
			Global.runTextIn.Text = "InputText: ";
			Global.runTextIn.SelectionStart = Global.runTextIn.Text.Length;
			ClearOutput();
			int ret = Algo358S.Algo358Smain(it);
			Global.runTextIn.Focus();
		} // Run

		private void buttonCalculate_Click(object sender, EventArgs e)
		// calculate with input from the text input box
		{
			buttonCalculate_Click();
		}

		private void buttonCalculate_Click()		// calculate with input from the text input box
		{
			Global.runTextIn.Focus();
			string st = Global.runTextIn.Text;		// get command input
			st = st.Trim();							// trim leading and trailing white space
			ClearInput();
			if (st != "")
				Calculate(ref st);
		}

		public void Calculate(ref string st) // calculate with input from string st
		{
			int ret;
			int iTc = 1;
			bool bTc;
			int at = st.IndexOf(":");
			if (at >= 0) st = st.Remove(0, at + 1);
			st = st.Trim();					// one final trim
			if (Global.IsInt32(st))
			{
				iTc = Convert.ToInt32(st);
				bTc = (1 <= iTc) && (iTc <= 2);
			}
			else
				bTc = false;
			if (bTc)
				ret = Algo358S.Algo358Smain(iTc);
			Global.runTextIn.Focus();
		} // Calculate

		public static void FocusBothUs()	// focus on input and output Run text boxes
		{
			Global.runTextOut.Select(Global.runTextOut.Text.Length, 0);	// move to bottom/end of output text box
			Global.runTextOut.Focus();
			Global.runTextIn.Focus();	// move to Run input text box
		}

		private void buttonClearLogFile_Click(object sender, EventArgs e)
		// clear and initialize the log file
		{
			Global.ClearLog();
			SetLogCaption();
			FocusBothUs();
		}

		private void buttonClearInput_Click(object sender, EventArgs e)	// clear Input Screen
		{
			ClearInput();
		}

		private void buttonClearOutput_Click(object sender, EventArgs e)	// clear Output Screen
		{
			ClearOutput();
		}

		public static void ClearInput() //Clear Input Screen
		{
			Global.runTextIn.Text = "";
			Global.runTextIn.Focus();
		}

		public static void ClearOutput() // clear Output Screen
		{
			Global.runTextOut.Text = "";
			Global.runTextIn.Focus();
		}

		private void buttonQuit_Click(object sender, EventArgs e)
		// quit the Run form and go to Startup form
		{
			tQuitFlag = false;
			this.Close(); // memberFormClosing will do the work
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
		// quit the Run form and go to Startup form
		{
			tQuitFlag = true;
			this.Close(); // memberFormClosing will do the work
		}

		private void richTextIn_TextChanged(object sender, EventArgs e)
		// accept command input if linefeed at end of line
		// but not for continuation lines
		{
			string st = Global.runTextIn.Text.Trim(' '); // get command input
			int n = st.Length;
			if (n > 0)
			{
				if (st[n - 1] == '\n')
				{
					if (n > 1)
					{
						if (st[n - 2] == '-' || st[n - 2] == '+')
						{
							return;
						}
					}
					buttonCalculate_Click();
				}
			}
		} // richTextIn_TextChanged

		private void RunForm_Resize(object sender, EventArgs e) // resize the run form
		{
			int jTop = buttonQuit.Top + buttonQuit.Height + 18;
			int topHeight = 98;
			this.richTextIn.SetBounds(1, jTop, this.ClientRectangle.Width - 2, topHeight);
			this.richTextOut.SetBounds(1, jTop + topHeight + 2, this.ClientRectangle.Width - 2, this.ClientRectangle.Height - jTop - topHeight - 3);
		}

		public void SetLogCaption() // set log command button Caption
		{
			if (Global.logg)
			{
				if (buttonLog.Text != "Logging Is On")
					buttonLog.Text = "Logging Is On";
			}
			else
			{
				if (buttonLog.Text != "Logging Is Off")
					buttonLog.Text = "Logging Is Off";
			}
		}

		private void FunctionKeys(object sender, System.Windows.Forms.KeyEventArgs e)
		// process requests from Function keys
		{
			if (e.KeyCode == Keys.F1)			// display the Help form
			{
				HelpForm();
			}
			else if (e.KeyCode == Keys.F2)		// quit program
			{
					TotalQuit();				// F2 for quit and exit program
			}
			else if (e.KeyCode == Keys.F8)		// accept input and Calculate
			{
				buttonCalculate_Click();
			}
			else if (e.KeyCode == Keys.F9)		// toggle Logging OR clear Log file
			{
				if (Control.ModifierKeys != Keys.Control)
				{
					buttonLog_Click(sender, e);	// F9 toggle Logging to Log file on/off
				}
				else
				{
					buttonClearLogFile_Click(sender, e);	// Ctrl+F9 clear Log file
				}
			}
			else if (e.KeyCode == Keys.F10)		// clear input TextBox
			{
				buttonClearInput_Click(sender, e);
			}
			else if (e.KeyCode == Keys.F11)		// clear output TextBox
			{
				buttonClearOutput_Click(sender, e);
			}
			else if (e.KeyCode == Keys.Escape)
			{
				ClearInput();
			}
		} // FunctionKeys

		private void richTextIn_KeyDown(object sender, KeyEventArgs e)
		{
			FunctionKeys(sender, e);
		}

		private void richTextOut_KeyDown(object sender, KeyEventArgs e)
		{
			FunctionKeys(sender, e);
		}

		private void helpFormF1ToolStripMenuItem_Click(object sender, EventArgs e)
		// display the Help form
		{
			HelpForm();
		}

		public static void HelpForm()	// bring up the Help form
		{
			if (Global.helpUp < 3)				// allow at most 3 Help Forms up
			{
				HelpForm objHelpForm = new HelpForm();
				Global.runTextIn.Focus();		// restore Focus to Text input window
				objHelpForm.Show();
				objHelpForm.Help();				// call sub Help in the Help Form
			}
		}

		public void TotalQuit() // check on request to totally Quit
		{
			if (Global.tExitFlag)
			{
				tQuitFlag = true;
				this.Close(); // memberFormClosing will do the work
			}
			else
			{
				DialogResult key = MessageBox.Show(
					"Do you wish to totally Quit " + Global.pName + "?     \n\n" +
					"To Quit select OK or Hit Enter\n\n" +
					"To Cancel Quit select Cancel or\n" +
					"      Hit the ESC key\n",
					Global.title,
					MessageBoxButtons.OKCancel,
					MessageBoxIcon.Information); // display message.
				// ----------
				if (key == DialogResult.OK)
				{
					tQuitFlag = true;
					this.Close(); // memberFormClosing will do the work
				}
				Global.runTextIn.Focus();
			}
		} // TotalQuit

		private void RunForm_FormClosing(object sender, FormClosingEventArgs e)
		// the Run form is being closed so quit the Run form and go to Startup form
		// with .exe file, Close button click while running giver "Illegal operation"
		// so I put in a running test and a cancel
		{
			if (Global.logg)
			{
				DateTimePicker dateTime = new System.Windows.Forms.DateTimePicker();
				string dateTimeString = dateTime.Value.ToString();
				Global.DoubleSpace();
				Global.WriteLn("Log file " + Global.logFile + " Closed " + dateTimeString);
				Global.WriteLn(null, Global.outL);
				Global.outL.Close();
				Global.logg = false;
			}
			this.Hide();
			Global.runFormUp = false;
			if (tQuitFlag)
			{
				Dispose(true);
				objStartupForm.TotalQuit();
			}
			else
				objStartupForm.Show();
		} // RunForm_FormClosing

		private void menuItemAbout_Click(object sender, EventArgs e)
		// bring up information about this program
		{
			AboutForm aboutForm = new AboutForm();
			aboutForm.ShowDialog();
			if (aboutForm.DialogResult == DialogResult.Cancel)
			{
				aboutForm.Dispose();
			}
		}

		private void buttonLog_Click(object sender, EventArgs e)
		// toggle the log screen to log file option
		{
			Global.ToggleLogg();
			FocusBothUs();
		}

		private bool tQuitFlag;					// total quit/exit flag
		// tQuitFlag helps implement the requirement that a close or menuItemExit will totally quit the
		// program while a Quit will go to the Startup form
		private StartupForm objStartupForm;
		public static int helpHeight = 520;		// current height of the Help Form in pixels
		public static int helpWidth = 690;		// current width of the Help Form in pixels
		public static int runHeight = 564;		// current height of the Run Form in pixels
		public static int runWidth = 674;		// current width of the Run Form in pixels
	}
}