using System;
using System.Windows.Forms;

namespace MyNamespace
{
	public partial class StartupForm : Form
	{
		public StartupForm()
		{
			InitializeComponent();
			//
			//TODO: Add the constructor code here
			//
			Global.logFileB = "Algo358S.Log";
			startupTicks = 1;
			defaultTestCase = 1;
			SetDefault();
			this.Text = Global.title + " - Startup";
			Pr("");
			Pr("   " + Global.name1);
			Pr("   " + Global.name2);
			Pr("   " + Global.version);
			Pr("   " + Global.author);
			Pr("   " + Global.address);
			Pr("");
			Pr("Algo358S is a program with the following features:");
			Pr("");
			Pr("     Tests the CSVD routine, ACM Algorithm 358 in C#.");
			Pr("");
			Pr("     Computes the Singular Value Decomposition of an m by n");
			Pr("     complex matrix a, which gives u as an m by m unitary matrix, s");
			Pr("     an n by n diagonal matrix and u an n by n unitary matrix, such");
			Pr("     that a = u * s * Conj(Tran(v)). The elements of s will be");
			Pr("     nonnegative and occur on the diagonal in descending order.");
			Pr("");
			Pr("     See: http://www.library.cornell.edu/nr/bookcpdf/c2-6.pdf");
			Pr("     Numerical Recipes Books On-Line.");
			Pr("");
			Pr("     See:");
			Pr("http://www.scs.fsu.edu/~burkardt/f77_src/toms358/toms358.f");
			Pr("http://www.scs.fsu.edu/~burkardt/f77_src/toms358/toms358_prb.f");
			Pr("     for the FORTRAN code.");
			Pr("     ");
			if (Global.command != "")
			{
					textBoxTestCase.Text = Global.command;
					textBoxTestCase_TextChanged();
					if (bTc)
							buttonRun_Click();
			}
			else
				textBoxTestCase.Text = defaultTestCase.ToString();
		} // StartupForm

		private void Pr(string st)
		{
			richTextIntro.Text += "   " + st + "\n";
		}

		private void buttonExit_Click(object sender, EventArgs e)
		{
			Dispose(true);
		}

		private void textBoxTestCase_TextChanged(object sender, EventArgs e)
		// test case [1, 2]
		{
			textBoxTestCase_TextChanged();
		}

		private void textBoxTestCase_TextChanged()
		// test case [1, 2]
		{
			string sTc = textBoxTestCase.Text.TrimStart(' ');	// remove leading spaces
			textBoxTestCase.Text = sTc;
			if (Global.IsInt32(sTc))
			{
				int iTc = Convert.ToInt32(sTc);
				bTc = (1 <= iTc) && (iTc <= 2);
			}
			else
				bTc = false;
			int n = sTc.Length;
			if (n > 0 && sTc[n - 1] == ' ') // remove space at end
			{
				sTc = sTc.TrimEnd(' ');
				textBoxTestCase.Text = sTc;
				textBoxTestCase.SelectionStart = sTc.Length; // stay at end
			}
			SetFonts();
		} // textBoxTestCase_TextChanged

		private void buttonDefault_Click(object sender, EventArgs e)
		{
			SetDefault();
		}

		private void SetDefault()
		{
			textBoxTestCase.Text = defaultTestCase.ToString();
			buttonRun.Enabled = bTc = true;
		}

		private void buttonRun_Click(object sender, EventArgs e)
		// start running the program
		{
			buttonRun_Click();
		}

		private void buttonRun_Click() // start running the calculator program
		{
			int iTc = System.Convert.ToInt32(textBoxTestCase.Text);
			this.Hide();
			RunForm objRunForm = new RunForm();
			objRunForm.Show();
			new Algo358S();
			objRunForm.Run(iTc, this); // call sub Run in the form RunForm
		}

		private void SetFonts()
		{
			if (bTc)
				this.textBoxTestCase.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold);
			else
				this.textBoxTestCase.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular);
			buttonRun.Enabled = bTc;
		}

		public void TotalQuit()
		{
			Dispose(true);
		}

		private void timerStartup_Tick(object sender, EventArgs e)
		{
			if (startupTicks > 0) startupTicks++;
			if (Global.runFormUp) this.Hide();		// this is only needed when there command line arguments are input
			if (startupTicks > 100)
				timerStartup.Enabled = false;		// and then only needed once
		}

		private static bool bTc;				// Test case good flag
		private static int defaultTestCase;		// Test case default
		private static int startupTicks = 0;	// startupTicks helps implement the requirement that the
												// StartupForm is bypassed if command line arguments are present
	}
}